voice_dBFS_diff=2
voice_dBFS_sample_interval=100
voice_dBFS_human_var_min=5
voice_dBFS_human_max=-10
voice_recognition_div_human_min=0.5
voice_ppl_max_length=15
voice_ppl_min_length=5
voice_modal_particles_pattern="啊|嗯|呃|呢|这个|那个"
voice_length_per_count_min=5
voice_length_per_count_max=25
voice_recognition_duration_per_pause_count_min=10
voice_recognition_duration_per_pause_count_max=50
voice_pause_duration_ratio_min=0
voice_pause_duration_ratio_max=0.3
voice_repeat_one_count_min=0
voice_repeat_one_count_max=5
voice_ppl_min=100
voice_ppl_max=200
skip_word4ppl=['我叫','岁','我是']
start_word4ppl=['然后','最后','接着']
text_clean_for_ppl="啊|嗯|呃|！|，|？|。"
text_clean_for_modal_particles="！|，|？|。"

KEY_FRAME_INTERVAL = 1


heart_rate_duration=1000
heart_rate_windows=1
